/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2009 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    Foam::phenomRhoRac5EqModel

Description
    Class for solving Rho-Rac 5 equation phenomenological model
    
    It should be noted that IROCKI_= 1/(1+I_ROCK/(K_I*(1+[ATP]/K_ATP))).
    Thus, we write IROCKI for simplicity, but this is a function of both
    inhibitor concentration and ATP.

\*---------------------------------------------------------------------------*/

#include "phenomRhoRac5EqModel_I.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //
Foam::phenomRhoRac5EqModel::phenomRhoRac5EqModel(const dictionary& chemicalProperties)			
:

    aDIARho_(readScalar(chemicalProperties.lookup("aDIARho"))),
    aROCKRho_(readScalar(chemicalProperties.lookup("aROCKRho"))),
    vDIAa_(readScalar(chemicalProperties.lookup("vDIAa"))),
    vDIAi_(readScalar(chemicalProperties.lookup("vDIAi"))),
    kDIAi_(readScalar(chemicalProperties.lookup("kDIAi"))),
    vROCKa_(readScalar(chemicalProperties.lookup("vROCKa"))),
    vROCKi_(readScalar(chemicalProperties.lookup("vROCKi"))),
    kROCKi_(readScalar(chemicalProperties.lookup("kROCKi"))),
    vRhoGEF_(readScalar(chemicalProperties.lookup("vRhoGEF"))),
    kRhoGEF_(readScalar(chemicalProperties.lookup("kRhoGEF"))),
    vRhoGAP_(readScalar(chemicalProperties.lookup("vRhoGAP"))),
    kRhoGAP_(readScalar(chemicalProperties.lookup("kRhoGAP"))),
    vRacGEF_(readScalar(chemicalProperties.lookup("vRacGEF"))),
    kRacGEF_(readScalar(chemicalProperties.lookup("kRacGEF"))),
    vRacGAP_(readScalar(chemicalProperties.lookup("vRacGAP"))),
    kRacGAP_(readScalar(chemicalProperties.lookup("kRacGAP"))),
    kDIAa_(readScalar(chemicalProperties.lookup("kDIAa"))),
    kROCKa_(readScalar(chemicalProperties.lookup("kROCKa"))),
    vPAKa_(readScalar(chemicalProperties.lookup("vPAKa"))),
    kPAKa_(readScalar(chemicalProperties.lookup("kPAKa"))),
    vPAKi_(readScalar(chemicalProperties.lookup("vPAKi"))),
    kPAKi_(readScalar(chemicalProperties.lookup("kPAKi"))),
    aRhoDIA_(readScalar(chemicalProperties.lookup("aRhoDIA"))),
    kRhoDIA_(readScalar(chemicalProperties.lookup("kRhoDIA"))),
    aRacDIA_(readScalar(chemicalProperties.lookup("aRacDIA"))),
    kRacDIA_(readScalar(chemicalProperties.lookup("kRacDIA"))),
    aRacROCK_(readScalar(chemicalProperties.lookup("aRacROCK"))),
    kRacROCK_(readScalar(chemicalProperties.lookup("kRacROCK"))),
    kDIARho_(readScalar(chemicalProperties.lookup("kDIARho"))),
    kROCKRho_(readScalar(chemicalProperties.lookup("kROCKRho"))),
    aRhoRac_(readScalar(chemicalProperties.lookup("aRhoRac"))),
    kRhoRac_(readScalar(chemicalProperties.lookup("kRhoRac"))),
    aRhoPAK_(readScalar(chemicalProperties.lookup("aRhoPAK"))),
    kRhoPAK_(readScalar(chemicalProperties.lookup("kRhoPAK"))),
    aPAKRac_(readScalar(chemicalProperties.lookup("aPAKRac"))),
    kPAKRac_(readScalar(chemicalProperties.lookup("kPAKRac"))),
    aRacPAK_(readScalar(chemicalProperties.lookup("aRacPAK"))),
    kRacPAK_(readScalar(chemicalProperties.lookup("kRacPAK"))),
    IPAKI_(readScalar(chemicalProperties.lookup("IPAKI"))),
    IROCKI_(readScalar(chemicalProperties.lookup("IROCKI"))),

    coeffs_(10, 0.0)
{}

// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //
Foam::phenomRhoRac5EqModel::~phenomRhoRac5EqModel()
{}

// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //



void Foam::phenomRhoRac5EqModel::derivatives		// right hand side of equations
(
    const scalar time,
    const scalarField& c,
    scalarField& dcdt
) const
{
    // RhoA equations
    dcdt[0] = - vRhoGEF_*((1.0 + aRhoDIA_*c[3]/kRhoDIA_) / (1.0 + c[3]/kRhoDIA_))*((1.0 + aRhoPAK_*c[9]/kRhoPAK_) / (1.0 + c[9]/kRhoPAK_))*(c[0]/kRhoGEF_)/(1.0+(c[0]/kRhoGEF_)) + vRhoGAP_*(c[1]/kRhoGAP_)/(1.0+(c[1]/kRhoGAP_));
    dcdt[1] = vRhoGEF_*((1.0 + aRhoDIA_*c[3]/kRhoDIA_) / (1.0 + c[3]/kRhoDIA_))*((1.0 + aRhoPAK_*c[9]/kRhoPAK_) / (1.0 + c[9]/kRhoPAK_))*(c[0]/kRhoGEF_)/(1.0+(c[0]/kRhoGEF_)) - vRhoGAP_*(c[1]/kRhoGAP_)/(1.0+(c[1]/kRhoGAP_));
    // DIA equations
    dcdt[2] = - vDIAa_*((1.0 + aDIARho_*c[1]/kDIARho_) / (1.0 + c[1]/kDIARho_))*(c[2]/kDIAa_)/(1.0+(c[2]/kDIAa_)) + vDIAi_*(c[3]/kDIAi_)/(1.0+(c[3]/kDIAi_));
    dcdt[3] = vDIAa_*((1.0 + aDIARho_*c[1]/kDIARho_) / (1.0 + c[1]/kDIARho_))*(c[2]/kDIAa_)/(1.0+(c[2]/kDIAa_)) - vDIAi_*(c[3]/kDIAi_)/(1.0+(c[3]/kDIAi_));
    // ROCK equations
    dcdt[4] = - vROCKa_*((1.0 + aROCKRho_*c[1]/kROCKRho_) / (1.0 + c[1]/kROCKRho_))*(c[4]/kROCKa_)/(1.0+(c[4]/kROCKa_)) + vROCKi_*(c[5]/kROCKi_)/(1.0+(c[5]/kROCKi_));
    dcdt[5] = vROCKa_*((1.0 + aROCKRho_*c[1]/kROCKRho_) / (1.0 + c[1]/kROCKRho_))*(c[4]/kROCKa_)/(1.0+(c[4]/kROCKa_)) - vROCKi_*(c[5]/kROCKi_)/(1.0+(c[5]/kROCKi_));
    // Rac1 equations
    dcdt[6] = - vRacGEF_*((1.0 + aRacDIA_*c[3]/kRacDIA_) / (1.0 + c[3]/kRacDIA_))*((1.0 + aRacPAK_*c[9]/kRacPAK_) / (1.0 + c[9]/kRacPAK_))*(c[6]/kRacGEF_)/(1.0+(c[6]/kRacGEF_)) + vRacGAP_*((1.0 + aRacROCK_*c[5]/(kRacROCK_*(1.0 + IROCKI_))) / (1.0 + c[5]/(kRacROCK_*(1.0 + IROCKI_))))*(c[7]/kRacGAP_)/(1.0+(c[7]/kRacGAP_));
    dcdt[7] =   vRacGEF_*((1.0 + aRacDIA_*c[3]/kRacDIA_) / (1.0 + c[3]/kRacDIA_))*((1.0 + aRacPAK_*c[9]/kRacPAK_) / (1.0 + c[9]/kRacPAK_))*(c[6]/kRacGEF_)/(1.0+(c[6]/kRacGEF_)) - vRacGAP_*((1.0 + aRacROCK_*c[5]/(kRacROCK_*(1.0 + IROCKI_))) / (1.0 + c[5]/(kRacROCK_*(1.0 + IROCKI_))))*(c[7]/kRacGAP_)/(1.0+(c[7]/kRacGAP_));
    // PAK equations
    dcdt[8] = - vPAKa_*((1.0 + aPAKRac_*c[7]/kPAKRac_) / (1.0 + c[7]/kPAKRac_))*(c[8]/(kPAKa_*(1.0 + IPAKI_)))/(1.0+(c[8]/(kPAKa_*(1.0 + IPAKI_)))) + vPAKi_*(c[9]/kPAKi_)/(1.0+(c[9]/kPAKi_));
    dcdt[9] = vPAKa_*((1.0 + aPAKRac_*c[7]/kPAKRac_) / (1.0 + c[7]/kPAKRac_))*(c[8]/(kPAKa_*(1.0 + IPAKI_)))/(1.0+(c[8]/(kPAKa_*(1.0 + IPAKI_)))) - vPAKi_*(c[9]/kPAKi_)/(1.0+(c[9]/kPAKi_));
}

void Foam::phenomRhoRac5EqModel::jacobian		// Jacobian - not needed for Runge-Kutta methods
(
    const scalar time,
    const scalarField& c,
    scalarField& dfdt,
    scalarSquareMatrix& dfdc
) const
{
}

void Foam::phenomRhoRac5EqModel::update(const scalar delta)
{}

void Foam::phenomRhoRac5EqModel::changeI		// function for changing of inhibitor concentrations, needed for hysteresis calculations
(
    const scalar IPAKI,
    const scalar IROCKI
) 
{
    IPAKI_ = IPAKI;
    IROCKI_ = IROCKI;
}


// ************************************************************************* //

